var map;
var currentMarker = null;

var form = document.getElementById("add-form");
var provinceField = form.querySelector("#province-field");
var provinceVal = new Choices(provinceField);

var regencyContainer = form.querySelector("#regency-container");
var regencyField = form.querySelector("#regency-field");
var regencyVal = new Choices(regencyField, {
    shouldSort: false,
});

var districtContainer = form.querySelector("#district-container");
var districtField = form.querySelector("#district-field");
var districtVal = new Choices(districtField, {
    shouldSort: false,
});

var mapContainer = form.querySelector("#map-container");

var loading = form.querySelector("#loading");

var btnContainer = form.querySelector("#btn-container");

document.addEventListener("DOMContentLoaded", function () {
    // Initialize the first map
    map = L.map("map", {
        attributionControl: false,
    }).setView([-8.157416852745705, 113.72281580436439], 16);

    // Add tile layers to both maps
    L.tileLayer("https://tile.openstreetmap.org/{z}/{x}/{y}.png", {
        maxZoom: 19,
    }).addTo(map);

    $("#showModal").on("shown.bs.modal", function () {
        map.invalidateSize();
    });
});

function validatePhoneNumber(input) {
    input.value = input.value.replace(/\D/g, "");

    if (input.value.length < 10 || input.value.length > 13) {
        input.setCustomValidity("Nomor telepon tidak valid");
    } else {
        input.setCustomValidity("");
    }
}

function getRegencies(provinceId) {
    regencyContainer.style.display = "none";
    districtContainer.style.display = "none";
    loading.style.display = "block";
    mapContainer.style.display = "none";

    const url = "/location/get-regency/" + provinceId;
    $.ajax({
        url: url,
        type: "GET",
        success: function (response) {
            if (response.success) {
                loading.style.display = "none";
                const data = response.data;

                regencyVal.clearStore();
                regencyVal.clearChoices();

                regencyVal.value = "";
                regencyVal.setChoices([
                    {
                        value: "",
                        label: "Pilih Kabupaten",
                        selected: true,
                        disabled: true,
                    },
                ]);

                if (Array.isArray(data)) {
                    regencyVal.setChoices(
                        data.map((regency) => ({
                            value: regency.id,
                            label: regency.name,
                            selected: false,
                            disabled: false,
                        }))
                    );
                    regencyContainer.style.display = "block";
                } else {
                    mapContainer.style.display = "block";
                    mapContainer.innerHTML =
                        "<p class='text-center text-muted'>Terjadi kesalaahan saat mengambil data, silahkan coba beberapa saat lagi</p>";
                }
            }
        },
        error: function (xhr, status, error) {
            mapContainer.style.display = "block";
            mapContainer.innerHTML =
                "<p class='text-center text-muted'>Terjadi kesalaahan saat mengambil data, silahkan coba beberapa saat lagi</p>";
        },
    });
}

function showmap(districtId) {
    if (!navigator.geolocation) {
        alert("Geolocation tidak didukung oleh browser ini.");
        return;
    }

    navigator.geolocation.getCurrentPosition(
        function (position) {
            mapContainer.style.display = "block";
            map.invalidateSize();
            btnContainer.style.display = "block";
            var lat = position.coords.latitude;
            var lng = position.coords.longitude;

            console.log(lat, lng);

            map.setView([lat, lng], 16);

            if (currentMarker) {
                map.removeLayer(currentMarker);
            }

            var iconMarker = L.icon({
                iconUrl: "/assets/images/marker.png",
                iconSize: [50, 50],
                iconAnchor: [25, 50],
                popupAnchor: [0, -50],
            });

            currentMarker = L.marker([lat, lng], {
                icon: iconMarker,
                draggable: true,
            }).addTo(map);

            currentMarker.bindPopup("Kamu berada di sini.").openPopup();
        },
        function (error) {
            alert("Error: " + error.message);
        }
    );
}

function getDistricts(regencyId) {
    districtContainer.style.display = "none";
    loading.style.display = "block";
    mapContainer.style.display = "none";

    const url = "/location/get-district/" + regencyId;
    $.ajax({
        url: url,
        type: "GET",
        success: function (response) {
            if (response.success) {
                loading.style.display = "none";
                const data = response.data;

                districtVal.clearStore();
                districtVal.clearChoices();

                districtVal.value = "";
                districtVal.setChoices([
                    {
                        value: "",
                        label: "Pilih Kecamatan",
                        selected: true,
                        disabled: true,
                    },
                ]);

                if (Array.isArray(data)) {
                    districtVal.setChoices(
                        data.map((district) => ({
                            value: district.id,
                            label: district.name,
                            selected: false,
                            disabled: false,
                        }))
                    );
                    districtContainer.style.display = "block";
                } else {
                    mapContainer.style.display = "block";
                    mapContainer.innerHTML =
                        "<p class='text-center text-muted'>Terjadi kesalaahan saat mengambil data, silahkan coba beberapa saat lagi</p>";
                }
            }
        },
        error: function (xhr, status, error) {
            mapContainer.style.display = "block";
            mapContainer.innerHTML =
                "<p class='text-center text-muted'>Terjadi kesalaahan saat mengambil data, silahkan coba beberapa saat lagi</p>";
        },
    });
}

function updateData(land) {
    var form = document.getElementById("edit-form");
    var landName = document.getElementById("landname-edit-field");
    var status = document.getElementById("status-edit-field");

    landName.value = land.name;
    status.value = land.description;
    form.action = "/data-lahan/" + land.id;
}

function deleteData(id) {
    var form = document.getElementById("delete-form");
    form.action = "/data-lahan/" + id;
}

var perPage = 10,
    options = {
        valueNames: ["id", "land_name", "status"],
        page: perPage,
        pagination: !0,
        plugins: [ListPagination({ left: 2, right: 2 })],
    },
    customerList = new List("customerList", options).on(
        "updated",
        function (e) {
            0 == e.matchingItems.length
                ? (document.getElementsByClassName(
                      "noresult"
                  )[0].style.display = "block")
                : (document.getElementsByClassName(
                      "noresult"
                  )[0].style.display = "none");
            var t = 1 == e.i,
                a = e.i > e.matchingItems.length - e.page;
            document.querySelector(".pagination-prev.disabled") &&
                document
                    .querySelector(".pagination-prev.disabled")
                    .classList.remove("disabled"),
                document.querySelector(".pagination-next.disabled") &&
                    document
                        .querySelector(".pagination-next.disabled")
                        .classList.remove("disabled"),
                t &&
                    document
                        .querySelector(".pagination-prev")
                        .classList.add("disabled"),
                a &&
                    document
                        .querySelector(".pagination-next")
                        .classList.add("disabled"),
                e.matchingItems.length <= perPage
                    ? (document.querySelector(
                          ".pagination-wrap"
                      ).style.display = "none")
                    : (document.querySelector(
                          ".pagination-wrap"
                      ).style.display = "flex"),
                e.matchingItems.length == perPage &&
                    document
                        .querySelector(".pagination.listjs-pagination")
                        .firstElementChild.children[0].click(),
                0 < e.matchingItems.length
                    ? (document.getElementsByClassName(
                          "noresult"
                      )[0].style.display = "none")
                    : (document.getElementsByClassName(
                          "noresult"
                      )[0].style.display = "block");
        }
    );
isCount = new DOMParser().parseFromString(
    customerList.items.slice(-1)[0]._values.id,
    "text/html"
);
var isValue = isCount.body.firstElementChild.innerHTML,
    idField = document.getElementById("id-field"),
    customerNameField = document.getElementById("customername-field"),
    emailField = document.getElementById("email-field"),
    dateField = document.getElementById("date-field"),
    phoneField = document.getElementById("phone-field"),
    statusField = document.getElementById("status-field"),
    addBtn = document.getElementById("add-btn"),
    editBtn = document.getElementById("edit-btn"),
    removeBtns = document.getElementsByClassName("remove-item-btn"),
    editBtns = document.getElementsByClassName("edit-item-btn");
function filterContact(e) {
    var t = e;
    customerList.filter(function (e) {
        matchData = new DOMParser().parseFromString(
            e.values().status,
            "text/html"
        );
        e = matchData.body.firstElementChild.innerHTML;
        return "All" == e || "All" == t || e == t;
    }),
        customerList.update();
}
function updateList() {
    var a = document.querySelector("input[name=status]:checked").value;
    (data = userList.filter(function (e) {
        var t = !1;
        return (
            "All" == a
                ? (t = !0)
                : ((t = e.values().sts == a), console.log(t, "statusFilter")),
            t
        );
    })),
        userList.update();
}
refreshCallbacks(),
    filterContact("All"),
    document
        .getElementById("showModal")
        .addEventListener("show.bs.modal", function (e) {
            e.relatedTarget.classList.contains("edit-item-btn")
                ? ((document.getElementById("exampleModalLabel").innerHTML =
                      "Edit Customer"),
                  (document
                      .getElementById("showModal")
                      .querySelector(".modal-footer").style.display = "block"),
                  (document.getElementById("add-btn").style.display = "none"),
                  (document.getElementById("edit-btn").style.display = "block"))
                : e.relatedTarget.classList.contains("add-btn")
                ? ((document.getElementById("exampleModalLabel").innerHTML =
                      "Tambah Data Lahan"),
                  (document
                      .getElementById("showModal")
                      .querySelector(".modal-footer").style.display = "block"),
                  (document.getElementById("edit-btn").style.display = "none"),
                  (document.getElementById("add-btn").style.display = "block"))
                : ((document.getElementById("exampleModalLabel").innerHTML =
                      "List Customer"),
                  (document
                      .getElementById("showModal")
                      .querySelector(".modal-footer").style.display = "none"));
        }),
    ischeckboxcheck(),
    document
        .getElementById("showModal")
        .addEventListener("hidden.bs.modal", function () {
            clearFields();
        }),
    document
        .querySelector("#customerList")
        .addEventListener("click", function () {
            refreshCallbacks(), ischeckboxcheck();
        });
var table = document.getElementById("customerTable"),
    tr = table.getElementsByTagName("tr"),
    trlist = table.querySelectorAll(".list tr"),
    count = Number(isValue.replace(/[^0-9]/g, "")) + 1;
addBtn.addEventListener("click", function (e) {
    "" !== customerNameField.value &&
        "" !== emailField.value &&
        "" !== dateField.value &&
        "" !== phoneField.value &&
        (customerList.add({
            id:
                '<a href="javascript:void(0);" class="fw-medium link-primary">#VZ' +
                count +
                "</a>",
            customer_name: customerNameField.value,
            email: emailField.value,
            date: dateField.value,
            phone: phoneField.value,
            status: isStatus(statusField.value),
        }),
        document.getElementById("close-modal").click(),
        clearFields(),
        refreshCallbacks(),
        filterContact("All"),
        count++);
}),
    // var statusVal = new Choices(statusField);
    function isStatus(e) {
        switch (e) {
            case "Active":
                return (
                    '<span class="badge badge-soft-success text-uppercase">' +
                    e +
                    "</span>"
                );
            case "Block":
                return (
                    '<span class="badge badge-soft-danger text-uppercase">' +
                    e +
                    "</span>"
                );
        }
    };
function ischeckboxcheck() {
    document.getElementsByName("checkAll").forEach(function (e) {
        e.addEventListener("click", function (e) {
            e.target.checked
                ? e.target.closest("tr").classList.add("table-active")
                : e.target.closest("tr").classList.remove("table-active");
        });
    });
}
function refreshCallbacks() {
    removeBtns.forEach(function (e) {
        e.addEventListener("click", function (e) {
            e.target.closest("tr").children[1].innerText,
                (itemId = e.target.closest("tr").children[1].innerText),
                customerList.get({ id: itemId }).forEach(function (e) {
                    deleteid = new DOMParser().parseFromString(
                        e._values.id,
                        "text/html"
                    );
                    var t = deleteid.body.firstElementChild;
                    deleteid.body.firstElementChild.innerHTML == itemId &&
                        document
                            .getElementById("delete-record")
                            .addEventListener("click", function () {
                                customerList.remove("id", t.outerHTML),
                                    document
                                        .getElementById("deleteRecordModal")
                                        .click();
                            });
                });
        });
    });
}
document
    .querySelector(".pagination-next")
    .addEventListener("click", function () {
        !document.querySelector(".pagination.listjs-pagination") ||
            (document
                .querySelector(".pagination.listjs-pagination")
                .querySelector(".active") &&
                document
                    .querySelector(".pagination.listjs-pagination")
                    .querySelector(".active")
                    .nextElementSibling.children[0].click());
    }),
    document
        .querySelector(".pagination-prev")
        .addEventListener("click", function () {
            !document.querySelector(".pagination.listjs-pagination") ||
                (document
                    .querySelector(".pagination.listjs-pagination")
                    .querySelector(".active") &&
                    document
                        .querySelector(".pagination.listjs-pagination")
                        .querySelector(".active")
                        .previousSibling.children[0].click());
        });
